@@ -10,8 +10,19 @@ from time import sleep
 @logged
 class ArticleDownloader:
 
-  def __init__(self, api_key):
-    self.headers = {'X-ELS-APIKEY': api_key}
+  def __init__(self, els_api_key=None, crf_api_key=None):
+    '''
+    Initialize and set up API keys
+
+    :param els_api_key: API key for Elsevier
+    :type els_api_key: str
+    :param crf_api_key: API key for CrossRef
+    :type crf_api_key: str
+    '''
+    self.headers = {
+      'X-ELS-APIKEY': els_api_key,
+      'CR-Clickthrough-Client-Token': crf_api_key
+      }
 
   @traced
   def check_els_entitlement(self, doi):
@@ -66,19 +77,25 @@ class ArticleDownloader:
     else: #Need to split queries
       stop_query = False
       cursor = '*'
+
       while not stop_query:
-        try:
-          search_url = base_url + query + '&rows=' + str(max_rows) + '&cursor=' + cursor
-          response = requests.get(search_url, headers=self.headers).json()
-          cursor = response['message']['next-cursor']
+        stop_ping = False
+        search_url = base_url + query + '&rows=' + str(max_rows) + '&cursor=' + cursor
 
-          if len(response["message"]["items"]) < max_rows or len(dois) >= rows:
-            stop_query = True
+        while not stop_ping:
+          response = requests.get(search_url, headers=self.headers)
+          try:
+            j_response = response.json()
+            cursor = j_response['message']['next-cursor']
+            stop_ping = True
+          except Exception, e:
+            self.__logger.warning(e)
 
-          for item in response["message"]["items"]:
-            dois.append(item["DOI"])
-        except:
-          return list(set(dois))
+        if len(j_response["message"]["items"]) < max_rows or len(dois) >= rows:
+          stop_query = True
+
+        for item in j_response["message"]["items"]:
+          dois.append(item["DOI"])
 
     return list(set(dois))
 
